@@ -18,6 +18,19 @@ use_python <- function(python, required = FALSE) {
   if (required && !file_test("-f", python) && !file_test("-d", python))
     stop("Specified version of python '", python, "' does not exist.")
 
+  # if required == TRUE and python is already initialized then confirm that we
+  # are using the correct version
+  if (required && is_python_initialized()) {
+    normalize <- function(path) {
+      normalizePath(normalize_python_path(path)$path, winslash = "/")
+    }
+    if (!identical(normalize(py_config()$python), normalize(python))) {
+      stop("Required version of Python '", python ,"' cannot be used ",
+           "because another version of Python ('", py_config()$python,
+           "') is already initialized ", "for this process.")
+    }
+  }
+
   if (required)
     .globals$required_python_version <- python
 
@@ -27,37 +40,33 @@ use_python <- function(python, required = FALSE) {
 
 #' @rdname use_python
 #' @export
-use_virtualenv <- function(virtualenv, required = FALSE) {
-
-  # prepend root virtualenv directory it doesn't exist and
-  # it's not an absolute path
-  if (!utils::file_test("-d", virtualenv) &
-      !grepl("^/|^[a-zA-Z]:/|^~", virtualenv, perl = TRUE)) {
-    workon_home <- Sys.getenv("WORKON_HOME", unset = "~/.virtualenvs")
-    virtualenv <- file.path(workon_home, virtualenv)
-  }
-
-  # compute the bin dir
-  if (is_windows())
-    python_dir <- file.path(virtualenv, "Scripts")
-  else
-    python_dir <- file.path(virtualenv, "bin")
+use_virtualenv <- function(virtualenv = NULL, required = FALSE) {
 
+  # resolve path to virtualenv
+  virtualenv <- virtualenv_path(virtualenv)
 
   # validate it if required
-  if (required && !is_python_virtualenv(virtualenv))
+  if (required && !is_virtualenv(virtualenv))
     stop("Directory ", virtualenv, " is not a Python virtualenv")
 
-  # set the option
-  python <- file.path(python_dir, "python")
-  if (is_windows())
-    python <- paste0(python, ".exe")
+  # get path to Python binary
+  suffix <- if (is_windows()) "Scripts/python.exe" else "bin/python"
+  python <- file.path(virtualenv, suffix)
   use_python(python, required = required)
+
 }
 
 #' @rdname use_python
 #' @export
-use_condaenv <- function(condaenv, conda = "auto", required = FALSE) {
+use_condaenv <- function(condaenv = NULL, conda = "auto", required = FALSE) {
+
+  # check for condaenv supplied by path
+  condaenv <- condaenv_resolve(condaenv)
+  if (grepl("[/\\]", condaenv, fixed = TRUE) && is_condaenv(condaenv)) {
+    python <- conda_python(condaenv)
+    use_python(python, required = required)
+    return(invisible(NULL))
+  }
 
   # list all conda environments
   conda_envs <- conda_list(conda)
